import { Injectable } from '@angular/core'
import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http'

import { Observable, forkJoin } from 'rxjs'
import { map } from 'rxjs/operators'

import { Apollo, gql } from 'apollo-angular'

import { WebsocketService } from '../_services/websocket.service'

const getLayerQuery = gql`
query mapboxStyle($store: String!) {
    mapboxStyle (store: $store) {
        paint
        layout
        attribution
    }
}
`

export class MapboxStyle {
    constructor(
        public paint: string,
        public layout: string,
        public attribution: string,
    ) {}
}


export class MapboxDataAndStyle {
    constructor(
        public data: object,
        public style: MapboxStyle,
    ) {}
}


@Injectable()
export class GeoJsonService {
    constructor(
        private _http: HttpClient,
        private apollo: Apollo
    ) {}

    getLayer(url: string, params?: object): Observable<object> {
        if (!params) {
            params = {}
        }
        return this._http.get<object>(url, {
            headers: <HttpHeaders>params,
        })
    }

    getStyle(store: string): Observable<MapboxStyle> {
        return this.apollo.query({
            query: getLayerQuery,
            variables: {
                store: store
            },
            errorPolicy: 'all',
        }).pipe(map(
            result => new MapboxStyle(
                result['data']['mapboxStyle']['paint'],
                result['data']['mapboxStyle']['layout'],
                result['data']['mapboxStyle']['attribution'],
            )
        ))
    }

    getAll(url: string, store: string, params?: object): Observable<MapboxDataAndStyle> {
        return forkJoin([
            this.getLayer(url, params),
            this.getStyle(store),
        ]).pipe(map(
            res => new MapboxDataAndStyle(res[0], res[1])
        ))
    }
}


@Injectable()
export class LiveGeoJsonService {
    messages = {}
    ws: any
    constructor(
        private wsService: WebsocketService
    ) {}

    connect(channel: string) {
        const hostname = window.location.hostname
        const port = window.location.port
        let protocol = window.location.protocol == 'https:' ? 'wss:' : 'ws:'
        this.wsService.connect(
            protocol + '//' + hostname + ':' + port + '/gj/live/' + channel
        )
        return this.wsService.ws
    }

    disconnect() {
        this.wsService.disconnect()
    }
}